from time import time, sleep
from threading import Thread
from dongle import NoData, CanError
import logging
from math import isnan

def ifbu(in_bytes): return int.from_bytes(in_bytes, byteorder='big', signed=False)
def ifbs(in_bytes): return int.from_bytes(in_bytes, byteorder='big', signed=True)
def ffbu(in_bytes): return float(int.from_bytes(in_bytes, byteorder='big', signed=False))
def ffbs(in_bytes): return float(int.from_bytes(in_bytes, byteorder='big', signed=True))

class DataError(Exception): pass

class Car:
    def __init__(self, config, dongle, gps):
        self.log = logging.getLogger("EVNotiPi/Car")
        self.config = config
        self.dongle = dongle
        self.gps = gps

    def pollData(self):
    # initialize data with required fields; saves all those checks later
        data = {
                    'timestamp':    None,
                    # Base:
                    'SOC_BMS':      None,
                    'SOC_DISPLAY':  None,
                    # Extended:
                    'auxBatteryVoltage':        None,
                    'batteryInletTemperature':  None,
                    'batteryMaxTemperature':    None,
                    'batteryMinTemperature':    None,
                    'cumulativeEnergyCharged':  None,
                    'cumulativeEnergyDischarged':   None,
                    'charging':                 None,
                    'normalChargePort':         None,
                    'rapidChargePort':          None,
                    'dcBatteryCurrent':         None,
                    'dcBatteryPower':           None,
                    'dcBatteryVoltage':         None,
                    'soh':                      None,
                    'externalTemperature':      None,
                    # Location:
                    'latitude':     None,
                    'longitude':    None,
                    'speed':        None,
                    'fix_mode':     0,
                }
        try:
            self.readDongle(data)  # readDongle updates data inplace
            return data['SOC_DISPLAY']
        except CanError as e:
            self.log.warning(e)
            return -1
        except NoData:
            self.log.info("NO DATA")
            return -1
