#!/usr/bin/env python3

from gevent.monkey import patch_all; patch_all()
from subprocess import check_call, check_output
from time import sleep,time
import os
import sys
import signal
import sdnotify
import logging
from argparse import ArgumentParser
import paho.mqtt.client as mqtt

Systemd = sdnotify.SystemdNotifier()


parser = ArgumentParser(description='EVNotiPi')
parser.add_argument('-d', '--debug', dest='debug', action='store_true', default=False)
parser.add_argument('-c', '--config', dest='config', action='store', default='config.yaml')
args = parser.parse_args()
del parser

# load config
if os.path.exists(args.config):
    if args.config[-5:] == '.json':
        import json
        with open(args.config, encoding='utf-8') as config_file:
            config = json.loads(config_file.read())
    elif args.config[-5:] == '.yaml':
        import yaml
        with open(args.config, encoding='utf-8') as config_file:
            config = None
            # use the last document in config.yaml as config
            for c in yaml.load_all(config_file, Loader=yaml.SafeLoader):
                config = c
    else:
        raise Exception('Unknown config type')
else:
    raise Exception('No config found')

if args.debug:
    logging.basicConfig(level=logging.DEBUG)
elif 'loglevel' in config:
    logging.basicConfig(level=config['loglevel'])
else:
    logging.basicConfig(level=logging.INFO)
log = logging.getLogger("evSoc")

del args

# Load OBD2 interface module
if not "{}.py".format(config['dongle']['type']) in os.listdir('dongles'):
    raise Exception('Unsupported dongle {}'.format(config['dongle']['type']))

# Init ODB2 adapter
sys.path.insert(0, 'dongles')
exec("from {0} import {0} as DONGLE".format(config['dongle']['type']))
sys.path.remove('dongles')

if not "{}.py".format(config['car']['type']) in os.listdir('cars'):
    raise Exception('Unsupported car {}'.format(config['car']['type']))

sys.path.insert(0, 'cars')
exec("from {0} import {0} as CAR".format(config['car']['type']))
sys.path.remove('cars')




# Init dongle
dongle = DONGLE(config['dongle'])

# Init car
car = CAR(config['car'], dongle, None)

# Set up signal handling
def exit_gracefully(signum, frame):
    sys.exit(0)

def on_connect(client, userdata, flags, rc):
    client.subscribe("openWB/global/ChargeMode")
    client.subscribe("openWB/lp/1/boolPlugStat")
    log.info("MQTT subscribed")

def on_publish(client, userdata, mid):
   print("published "+userdata)

charging=False

def on_message(client, userdata, msg):
    global charging
    if (msg.topic == "openWB/global/ChargeMode"):
        if (int(msg.payload) == 3):			# Stopped
            log.info("Charging stopped")
            charging = False
        else:
            log.info("Charging started")
            charging = True
    if (msg.topic == "openWB/lp/1/boolPlugStat"):
        if (int(msg.payload) == 0):			# Stopped
            log.info("Unplugged")
            charging = False
        else:
            log.info("Plugged")
            charging = True

signal.signal(signal.SIGTERM, exit_gracefully)

mqtt_broker_ip = config['mqtt']['server']
client = mqtt.Client()

client.on_connect=on_connect
client.on_message=on_message
#client.on_publish=on_publish

try:
    client.connect(mqtt_broker_ip, config['mqtt']['port'])
except:
    log.error("MQTT-connect failed")

Systemd.notify("READY=1")

client.loop_start()

try:
    while(True):
        if(charging==True):
            soc=car.pollData()
            if(soc<0):
                log.warning("Re-Init Dongle")
                dongle.initDongle()
            else:
                log.info("SoC=" + str(soc))
                client.publish(config['mqtt']['topic'], round(soc), qos=0, retain=True)
        sleep(10)

except (KeyboardInterrupt, SystemExit): #when you press ctrl+c
    main_running = False
    Systemd.notify("STOPPING=1")
finally:
    Systemd.notify("STOPPING=1")
    client.disconnect()
    log.info("Exiting ...")
    log.info("Bye.")

